"""
Description: \n
	Hello Random Coder. \n
	Here you'll find everything eventhough you're not supposed to. \n
	\n
	This is the "Core Engine" if you will. \n
\n
======================================================== \n
File Created on:		2024 Jan. 11 \n
File Created by:		Simon Arjuna Erat \n
License:				MIT \n
URL:					https://www.github.com/sri-arjuna/ASPIRE \n
Based on my TUI & SWARM for the BASH shell © 2011 \n
"""
